import { db } from "@/lib/db"
import fetchUserUsingAPI from "@/lib/validate-api";
import { NextResponse } from "next/server";

export async function GET(req: Request, { params } : { params: { id: string } }) {
    try {
        let id = req.headers?.get('x-api-key')
        if (id == null) {
          return NextResponse.json(null);
        }

        const tokens = await db.apiKey.findFirst({
          where: {
            id: id
          }
        });

        return NextResponse.json(tokens);
    } catch (error) {
        console.log("[TOKEN/GET]", error);
        return new NextResponse("Internal Error", { status: 500});
    }
}

export async function DELETE(req: Request, { params } : { params: { id: string } }) {
    try {
        const { id } = params
        const user = await fetchUserUsingAPI(req)

        const token = await db.apiKey.delete({
          where: {
            id,
            userId: user?.id
          }
        });

        return NextResponse.json(token);
    } catch (error) {
        console.log("[TOKEN/DELETE]", error);
        return new NextResponse("Internal Error", { status: 500});
    }
}